/*
* file CcActivity.java
*
* Licensed Materials - Property of IBM
* Restricted Materials of IBM
* 
* com.ibm.rational.wvcm.stp.cc.CcActivity
*
*  Copyright IBM Corporation 2004, 2008.  All Rights Reserved. 
* Note to U.S. Government Users Restricted Rights:  Use, duplication or  
* disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
*/

package com.ibm.rational.wvcm.stp.cc;

import javax.wvcm.Activity;
import javax.wvcm.Feedback;
import javax.wvcm.ResourceList;
import javax.wvcm.Version;
import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;

import com.ibm.rational.wvcm.stp.StpActivity;
import com.ibm.rational.wvcm.stpex.StpExBase;

/**
 * <p>A proxy for a ClearCase UCM activity.
 * </p>
 * <p>UCM activities collect the logically-related changes (versions) made in a
 * given UCM stream, and are the basic unit of <i>change flow </i> between
 * streams. When a developer begins working on a new task within a particular
 * UCM stream, he typically creates a new UCM activity for that task. At
 * checkout time, each new version is added to the activity's <i>change set</i>.
 * </p>
 * <p>If the UCM project in which the user working is <i>ClearQuest-enabled</i>,
 * all UCM activities in that project are managed by the <i>ClearQuest
 * Integration </i>. Rather than creating a new UCM activity directly, the user
 * selects or creates the ClearQuest record he wants to work on and performs a
 * <i>WorkOn </i> operation. ClearQuest creates a corresponding UCM activity and
 * binds it to the ClearQuest record.
 * </p>
 * <p>See the "Developing Software with ClearCase" manual for more information
 * about working with UCM activities.
 * </p>
 */
public interface CcActivity
    extends Activity, StpActivity, CcVobResource
{
    /**
     * <p>Create a new UCM activity at the location identified by this proxy. The
     * location should be an object name selector specifying the activity's name
     * and the repository (project VOB) in which to create it.
     * </p>
     * <p>Set the {@link #STREAM} property to specify the new activity's stream
     * (required). The stream's repository must match the repository specified
     * in this activity's location.
     * </p>
     * <p>Set the {@link #HEADLINE} property to specify the new activity's headline
     * (optional).
     * </p>
     * <p>Set the {@link #COMMENT} property to specify a creation comment for the
     * new activity (optional).
     * </p>
     * <p>This method fails if the stream is ClearQuest-enabled. In a CQ-enabled
     * stream, activities can only be created indirectly by working on a CQ
     * entity in the context of a view associated with that stream.
     * </p>
     */
    public CcActivity doCreateCcActivity(Feedback feedback) throws WvcmException;

    /**
     * <p>Create a new UCM activity, allowing the provider to choose the
     * new activity's name.
     * The provider may use the client-specified location if it is valid,
     * but can select a different location if the location is not valid
     * or already identifies an activity.
     * </p>
     * @see #doCreateCcActivity(Feedback)
     */
    public CcActivity doCreateGeneratedCcActivity(Feedback feedback) throws WvcmException;


    /** The UCM stream in which this activity resides. */
    static final PropertyName<CcStream> STREAM =
        new PropertyName<CcStream>(StpExBase.PROPERTY_NAMESPACE, "act-stream");

    /**
     * Get the value of this proxy's {@link #STREAM} property.
     * @return this activity's stream
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    CcStream getStream() throws WvcmException;

    /**
     * Set the value of this activity's {@link #STREAM} property.
     * This property can only be set at activity creation time.
     * @param stream proxy for the stream in which the new activity will reside
     */
    void setStream(CcStream stream);
    
    /**
     * Return true if the activity is an "integration activity", that is
     * one created by UCM during a deliver or rebase operation.
     */
    static final PropertyName<Boolean> IS_INTEGRATION_ACTIVITY =
        new PropertyName<Boolean>(StpExBase.PROPERTY_NAMESPACE, "is-integration-activity");

    /**
     * Get the value of this proxy's {@link #IS_INTEGRATION_ACTIVITY} property.
     * @return true if this proxy represents an integration activity, false otherwise
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    boolean getIsIntegrationActivity() throws WvcmException;

    /** A view for resolving names of versions in the activity's change set. */
    static final PropertyName<CcView> NAME_RESOLVER_VIEW =
        new PropertyName<CcView>(StpExBase.PROPERTY_NAMESPACE, "name-resolver-view");
 
    /**
     * Get the value of this proxy's {@link #NAME_RESOLVER_VIEW} property.
     * @return A view on the activity's stream.
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    CcView getNameResolverView() throws WvcmException;

    /** The latest version for each element in a change set. */
    static final PropertyName<ResourceList<Version>> LATEST_VERSION_LIST =
        new PropertyName<ResourceList<Version>>(StpExBase.PROPERTY_NAMESPACE, "latest-version-list");
 
    /**
     * Get the value of this proxy's {@link #LATEST_VERSION_LIST} property.
     * @return The latest version of each element in a change set.
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    ResourceList<Version> getLatestVersionList() throws WvcmException;
    
    /** Returns true if there are checkouts under this activity. */
    static final PropertyName<Boolean> HAS_CHECKOUTS = 
        new PropertyName<Boolean>(StpExBase.PROPERTY_NAMESPACE, "has-checkouts");
    
    /**
     * Get the value of this proxy's {@link #HAS_CHECKOUTS} proerty.
     * @return true if there are checkouts under the activity represented by this proxy, false otherwise.
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    boolean getHasCheckouts() throws WvcmException;
}
